import os, shutil
import unittest
from StringIO import StringIO
from urllib import splitquery

from limewire.filter import outputfilter
from limewire import filter

class Table(dict):
    def add(self, key, value):
        self.__setitem__(key, value)

class Request:
    def __init__(self, method, uri=''):
        self.headers_in = Table()
        self.headers_out = Table()
        self.main = None
        self.method = method
        host, query = splitquery(uri)
        self.parsed_uri = (None, None, None, None, None, None, None, query)
        
class Filter:
    def __init__(self, input, output, is_last=True, method='GET', uri=''):
        self.input = input
        self.output = output
        self.is_closed = False
        self.req = Request(method, uri)
        self.is_last = is_last
        
    def read(self, length=None):
        data = self.input.read()
        if data == '' and self.is_last:
            return None
        return data
        
    def readline(self, length=None):
        data = self.input.readline()
        if data == '' and self.is_last:
            return None
        return data
    
    def write(self, data):
        self.output.write(data)
        
    def close(self):
        self.is_closed = True

    def disable(self):
        pass

    def pass_on(self):
        data = self.input.read()
        self.output.write(data)

    def flush(self):
        pass


class LimewireTests(unittest.TestCase):
    def setUp(self):
        # just a hack so we get a db file somewhere
        filter.DBFILE = os.tmpnam()

    def tearDown(self):
        # I'm assuming .db on the end, this may be platform specific
        os.remove("%s.db" % filter.DBFILE)

    def test_outputfilter(self):

        input = StringIO()
        output = StringIO()

        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:12345")
        outputfilter(filter)

        self.assertEquals(filter.req.headers_out.get('X-Node'), None)

    def test_outputfilter_xnode1(self):

        input = StringIO()
        output = StringIO()

        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:12345")
        filter.req.headers_in['x-node'] = '192.168.1.1:8000'
        outputfilter(filter)

        self.assertEquals(filter.req.headers_out.get('x-alt'), None)

    def test_outputfilter_xnode2(self):

        input = StringIO()
        output = StringIO()

        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:12345")
        filter.req.headers_in['x-node'] = '192.168.1.1:8000'
        outputfilter(filter)

        self.assertEquals(filter.req.headers_out.get('x-alt'), None)

    def test_outputfilter_xnode3(self):

        input = StringIO()
        output = StringIO()

        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:12345")
        filter.req.headers_in['x-node'] = '192.168.1.1:8000'
        outputfilter(filter)

        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:12345")
        filter.req.headers_in['x-node'] = '192.168.1.2:8000'
        outputfilter(filter)

        self.assertEquals(filter.req.headers_out.get('x-alt'), '192.168.1.1:8000')

    def test_outputfilter_xnode4(self):

        input = StringIO()
        output = StringIO()

        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:12345")
        filter.req.headers_in['x-node'] = '192.168.1.1:8000'
        outputfilter(filter)

        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:12345")
        filter.req.headers_in['x-node'] = '192.168.1.2:8000'
        outputfilter(filter)

        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:12345")
        filter.req.headers_in['x-node'] = '192.168.1.3:8000'
        outputfilter(filter)

        self.assertEquals(filter.req.headers_out.get('x-alt'), '192.168.1.1:8000,192.168.1.2:8000')

    def test_outputfilter_xnode5(self):

        input = StringIO()
        output = StringIO()

        # lets try with differnt sha1 hash, should be empty again
        filter = Filter(input, output, uri="/uri-res/N2R?urn:sha1:67890")

        filter.req.headers_in['x-node'] = '192.168.1.1:8000'

        outputfilter(filter)

        self.assertEquals(filter.req.headers_out.get('x-alt'), None)


def get_data_path(name):
    path, rest = os.path.split(__file__)
    return os.path.join(path, 'data', name)

    
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(LimewireTests))
    return suite
